import sys

from os import environ

from os.path import abspath, dirname

# this only makes sence while i need the lib folder in the PYTHONPATH
# otherwise i need to rewrite many code in comparison to how i need it
# to be able to change things more easily and work on the code
__lib = '%s/lib'%abspath(dirname(__file__))
if not __lib in sys.path:
	sys.path = [__lib] + sys.path

from colortext import abort, fatal

from pwclip.cmdline import cli

def pwclipper():
	try:
		cli()
	except RuntimeError as err:
		fatal(err)
	except KeyboardInterrupt:
		abort



#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
#
# This file is free software by d0n <d0n@janeiskla.de>
#
# You can redistribute it and/or modify it under the terms of the GNU -
# Lesser General Public License as published by the Free Software Foundation
#
# This is distributed in the hope that it will be useful somehow.
#
# !WITHOUT ANY WARRANTY!
#
# Without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
"""
clips - clipboard for various systems
"""
import sys

from os import name as osname, environ

from platform import system

from time import sleep, time

from subprocess import Popen, PIPE

def clips():
	"""return `copy`, `paste` as system independent functions"""
	def winclips():
		"""windows clipboards - the ugliest thing i've ever seen"""
		from ctypes import \
            windll, memmove, \
            c_size_t, sizeof, \
            c_wchar_p, get_errno, c_wchar
		from ctypes.wintypes import \
            INT, HWND, DWORD, \
            LPCSTR, HGLOBAL, LPVOID, \
            HINSTANCE, HMENU, BOOL, UINT, HANDLE
		from contextlib import contextmanager
		GMEM_MOVEABLE = 0x0002
		CF_UNICODETEXT = 13
		class CheckedCall(object):
			def __init__(self, f):
				super(CheckedCall, self).__setattr__("f", f)
			def __call__(self, *args):
				ret = self.f(*args)
				if not ret and get_errno():
					raise PyperclipWindowsException("Error calling " + self.f.__name__)
				return ret
			def __setattr__(self, key, value):
				setattr(self.f, key, value)
		mkwin = CheckedCall(windll.user32.CreateWindowExA)
		mkwin.argtypes = [
            DWORD, LPCSTR,
            LPCSTR, DWORD,
            INT, INT,
            INT, INT,
            HWND, HMENU,
            HINSTANCE, LPVOID]
		mkwin.restype = HWND
		delwin = CheckedCall(windll.user32.DestroyWindow)
		delwin.argtypes = [HWND]
		delwin.restype = BOOL
		clip = windll.user32.OpenClipboard
		clip.argtypes = [HWND]
		clip.restype = BOOL
		clsclip = CheckedCall(windll.user32.CloseClipboard)
		clsclip.argtypes = []
		clsclip.restype = BOOL
		delclip = CheckedCall(windll.user32.EmptyClipboard)
		delclip.argtypes = []
		delclip.restype = BOOL
		getclip = CheckedCall(windll.user32.GetClipboardData)
		getclip.argtypes = [UINT]
		getclip.restype = HANDLE
		setclip = CheckedCall(windll.user32.SetClipboardData)
		setclip.argtypes = [UINT, HANDLE]
		setclip.restype = HANDLE
		allock = CheckedCall(windll.kernel32.GlobalAlloc)
		allock.argtypes = [UINT, c_size_t]
		allock.restype = HGLOBAL
		dolock = CheckedCall(windll.kernel32.GlobalLock)
		dolock.argtypes = [HGLOBAL]
		dolock.restype = LPVOID
		unlock = CheckedCall(windll.kernel32.GlobalUnlock)
		unlock.argtypes = [HGLOBAL]
		unlock.restype = BOOL
		@contextmanager
		def window():
			"""
			Context that provides a valid Windows hwnd.
			"""
			hwnd = mkwin(0, b"STATIC", None, 0, 0, 0, 0, 0,
									   None, None, None, None)
			try:
				yield hwnd
			finally:
				delwin(hwnd)
		@contextmanager
		def clipboard(hwnd):
			"""
			Context manager that opens the clipboard and prevents
			other applications from modifying the clipboard content.
			"""
			t = time() + 0.5
			success = False
			while time() < t:
				success = clip(hwnd)
				if success:
					break
				sleep(0.01)
			if not success:
				raise Exception("could not open clipboard")
			try:
				yield
			finally:
				clsclip()
		def _copy(text):
			with window() as hwnd:
				with clipboard(hwnd):
					delclip()
					if text:
						count = len(text) + 1
						handle = allock(GMEM_MOVEABLE,
												 count * sizeof(c_wchar))
						locked_handle = dolock(handle)
						ctypes.memmove(c_wchar_p(locked_handle),
							c_wchar_p(text), count * sizeof(c_wchar))
						unlock(handle)
						setclip(CF_UNICODETEXT, handle)
		def _paste():
			with clipboard(None):
				handle = getclip(CF_UNICODETEXT)
				if not handle:
					return ""
				return c_wchar_p(handle).value

		return _copy, _paste

	def osxclips():
		def _copy(text):
			text = text if text else ''
			p = Popen(['pbcopy', 'w'], stdin=subprocess.PIPE, close_fds=True)
			p.communicate(input=text.encode('utf-8'))
		def _paste():
			p = subprocess.Popen(['pbpaste', 'r'],
                    stdout=subprocess.PIPE, close_fds=True)
			out, _ = p.communicate()
			return out.decode('utf-8')
		return _copy, _paste

	def linclips():
		"""linux clipboards"""
		def _copy(text, mode='p'): # mode in ('p', 'b', 'pb')
			"""linux copy function"""
			text = text if text else ''
			if 'p' in mode:
				with Popen([
                      'xsel', '-l', '/dev/null', '-p', ], stdin=PIPE) as prc:
					prc.communicate(input=text.encode('utf-8'))
			if 'b' in mode:
				with Popen([
                      'xsel', '-l', '/dev/null', '-b', ], stdin=PIPE) as prc:
					prc.communicate(input=text.encode('utf-8'))

		def _paste(mode='p'):
			"""linux paste function"""
			if mode == 'p':
				out, _ = Popen([
                    'xsel', '-l', '/dev/null', '-p', '-o'
                    ], stdout=PIPE).communicate()
				return out.decode()
			elif mode == 'b':
				out, _ = Popen([
                    'xsel', '-l', '/dev/null', '-b', '-o'
                    ], stdout=PIPE).communicate()
				return out.decode()
			elif mode in ('pb', 'bp'):
				pout, _ = Popen([
                    'xsel', '-l', '/dev/null', '-p', '-o'
                    ], stdout=PIPE).communicate()
				bout, _ = Popen([
                    'xsel', '-l', '/dev/null', '-b', '-o'
                    ], stdout=PIPE).communicate()
				return pout.decode(), bout.decode()
		return _copy, _paste
	# decide which copy, paste functions to return [windows|mac|linux] mainly
	if osname == 'nt' or system() == 'Windows':
		return winclips()
	elif osname == 'mac' or system() == 'Darwin':
		return osxclips()
	return linclips()

copy, paste = clips()



#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
"""
gpgtool module
"""

# (std)lib imports
from os import X_OK, access, getcwd, environ, path, name as osname

from getpass import getpass

from psutil import process_iter as piter

from tkinter import TclError

from getpass import getpass

from gnupg import GPG

# local imports
from colortext import blu, red, yel, bgre, tabd, abort, error, fatal

from system import xinput, xyesno, xmsgok



class GPGTool(object):
	"""
	gnupg wrapper-wrapper :P
	although the gnupg module is quite handy and the functions are pretty and
	useable i need some modificated easing functions to be able to make the
	main code more easy to understand by wrapping multiple gnupg functions to
	one - also i can prepare some program related stuff in here
	"""
	_dbg = None
	homedir = path.join(path.expanduser('~'), '.gnupg')
	__bindir = '/usr/bin'
	__gpgbin = 'gpg2'
	if osname == 'nt':
		__bindir = 'C:\Program Files (x86)\GNU\GnuPG'
		__gpgbin = 'gpg2.exe'
	binary = path.join(__bindir, __gpgbin)
	if not path.isfile(binary) or not access(binary, X_OK):
		raise RuntimeError('%s needs to be executable'%binary)
	agentinfo = path.join(homedir, 'S.gpg-agent')
	kginput = {}
	__pin = None
	def __init__(self, *args, **kwargs):
		for arg in args:
			arg = '_%s'%arg
			if hasattr(self, arg):
				setattr(self, arg, True)
		for (key, val) in kwargs.items():
			if hasattr(self, key) and not isinstance(val, bool):
				setattr(self, key, val)
		if self.dbg:
			lim = int(max(len(k) for k in GPGTool.__dict__.keys()))+4
			print('%s\n%s\n\n%s\n%s\n'%(
                GPGTool.__mro__,
                '\n'.join('  %s%s=    %s'%(
                    k, ' '*int(lim-len(k)), v
                ) for (k, v) in sorted(GPGTool.__dict__.items())),
                GPGTool.__init__,
                '\n'.join('  %s%s=    %s'%(k, ' '*int(lim-len(k)), v
                ) for (k, v) in sorted(self.__dict__.items()))))
	@property                # dbg <bool>
	def dbg(self):
		"""bool"""
		return self._dbg
	@dbg.setter
	def dbg(self, val):
		self._dbg = bool(val)

	@property                # keyring <str>
	def keyring(self):
		__bin = self.binary.rstrip('.exe')
		return path.join(self.homedir, 'pubring.kbx') \
            if __bin.endswith('2') else path.join(self.homedir, 'pubring.gpg')

	@property                # secring <str>
	def secring(self):
		__bin = self.binary.rstrip('.exe')
		if __bin.endswith('2') and self.keyring.endswith('gpg'):
			return path.join(self.homedir, 'secring.gpg')
		elif not __bin.endswith('2'):
			return path.join(self.homedir, 'secring.gpg')
		return self.keyring

	@property                # _gpg_ <GPG>
	def _gpg_(self):
		"""object"""
		opts = ['--batch', '--always-trust']
		if osname != 'nt':
			opts.append('--pinentry-mode=loopback')
		if self.__pin: opts = opts + ['--passphrase=%s'%self.__pin]
		__g = GPG(
            keyring=self.keyring, secret_keyring=self.secring,
            gnupghome=self.homedir, gpgbinary=self.binary,
            use_agent=True, options=opts,
            verbose=1 if self.dbg else 0)
		__g.encoding = 'utf-8'
		return __g

	def _garr(self):
		for proc in piter():
			if proc.name() in ('gpg-agent', 'scdaemon', 'dirmngr'):
				try:
					proc.kill()
				except PermissionError:
					error(
                        'cannot kill process',
                        proc.name, 'with PID', proc.pid())
		gacfg = path.join(self.homedir, 'gpg-agent.conf')
		try:
			with open(gacfg, 'r') as afh:
				for l in afh.readlines():
					if 'enable-ssh-support' in l and not '#' in l:
						environ['GPG_AGENT_INFO'] = \
                            path.join(self.homedir, 'S.gpg-agent.ssh')
		except FileNotFoundError as err:
			error(err)
		environ['GPG_AGENT_INFO'] = path.join(self.homedir, 'S.gpg-agent')

	@staticmethod
	def _passwd(rpt=False):
		"""
		password questioning function
		"""
		msg = 'enter passphrase: '
		tru = 'repeat that passphrase: '
		while True:
			try:
				if not rpt:
					return getpass(msg)
				__pwd = getpass(msg)
				if __pwd == getpass(tru):
					return __pwd
				error('passwords did not match')
			except KeyboardInterrupt:
				abort()

	def genkeys(self, **kginput):
		"""
		gpg-key-pair generator method
		"""
		if self.dbg:
			print(bgre(self.genkeys))
		kginput = kginput if kginput != {} else self.kginput
		if not kginput:
			error('no key-gen input received')
			return
		print(
            blu('generating new keys using:\n '),
            '\n  '.join('%s%s=  %s'%(
                blu(k),
                ' '*int(max(len(s) for s in kginput.keys())-len(k)+2),
                yel(v)
            ) for (k, v) in kginput.items()))
		if 'passphrase' in kginput.keys():
			if kginput['passphrase'] == 'nopw':
				del kginput['passphrase']
			elif kginput['passphrase'] == 'stdin':
				kginput['passphrase'] = self.__passwd(rpt=True)
		print(red('generating %s-bit keys - this WILL take some time'%(
            kginput['key_length'])))
		key = self._gpg_.gen_key(self._gpg_.gen_key_input(**kginput))
		if self.dbg:
			print('key has been generated:\n%s'%str(key))
		return key

	@staticmethod
	def __find(pattern, *vals):
		for val in vals:
			if isinstance(val, (list, tuple)) and \
			      [v for v in val if pattern in v]:
				#print(val, pattern)
				return True
			elif pattern in val:
				#print(val, pattern)
				return True

	def findkey(self, pattern='', **kwargs):
		typ = 'A' if not 'typ' in kwargs.keys() else kwargs['typ']
		secret = False if not 'secret' in kwargs.keys() else kwargs['secret']
		keys = {}
		pattern = pattern if not pattern.startswith('0x') else pattern[2:]
		for key in self._gpg_.list_keys():
			if pattern and not self.__find(pattern, *key.values()):
				continue
			for (k, v) in key.items():
				#print(k, v)
				if k == 'subkeys':
					#print(k)
					for sub in key[k]:
						#print(sub)
						short, typs, finger = sub
						#print(finger, typs)
						if typ == 'A' or (typ in typs):
							si = key[k].index(sub)
							ki = key[k][si].index(finger)
							kstr = self._gpg_.export_keys(
                                key[k][si][ki], secret=secret)
							#print(kstr)
							keys[finger] = {typs: kstr}
		return keys

	def export(self, *patterns, **kwargs):
		"""
		key-export method
		"""
		if self.dbg:
			print(bgre(self.export))
		typ = 'A' if not 'typ' in kwargs.keys() else kwargs['typ']
		secret = False if not 'secret' in kwargs.keys() else kwargs['secret']
		keys = dict((k, v) for (k, v) in self.findkey(**kwargs).items())
		if patterns:
			keys = dict((k, v) for p in list(patterns) \
                for (k, v) in self.findkey(p, **kwargs).items())
		return keys

	def _encryptwithkeystr(self, message, keystr, output):
		fingers = [
            r['fingerprint'] for r in self._gpg_.import_keys(keystr).results]
		return self._gpg_.encrypt(
            message, fingers, always_trust=True, output=output)

	def encrypt(self, message, *args, **kwargs):
		"""
		text encrypting function
		"""
		if self.dbg:
			print(bgre(self.encrypt))
		fingers = list(self.export(**{'typ': 'e'}))
		if 'recipients' in kwargs.keys():
			fingers = list(self.export(*kwargs['recipients'], **{'typ': 'e'}))
		if 'keystr' in kwargs.keys():
			res = self._gpg_.import_keys(keystr).results[0]
			fingers = [res['fingerprint']]
		output = None if not 'output' in kwargs.keys() else kwargs['output']
		#print(fingers)
		return self._gpg_.encrypt(
            message, fingers, always_trust=True, output=output)

	def decrypt(self, message, output=None):
		"""
		text decrypting function
		"""
		if self.dbg:
			print(bgre('%s\n  trying to decrypt:\n%s'%(self.decrypt, message)))
		c = 0
		try:
			while True:
				__plain = self._gpg_.decrypt(
					message.strip(), always_trust=True, output=output)
				if __plain:
					return __plain
				if not __plain and c == 0:
					self._garr()
				if c > 3:
					try:
						xmsgok('too many wrong attempts')
					except TclError:
						input('too many wrong attempts')
					break
				elif c > 1 and c < 3:
					try:
						yesno = xyesno(
						  'decryption failed - try again?')
					except TclError:
						yesno = input(
							'no passphrase or no secret key, retry? [Y/n] ')
					if not yesno or yesno is not True and yesno.lower() == 'n':
						break
				elif c > 1 and not self.__pin:
					try:
						yesno = xyesno('no passphrase entered, retry?')
					except TclError:
						yesno = input(
							'no passphrase entered, retry? [Y/n] ')
					if yesno is False or yesno.lower() == 'n':
						break
				c+=1
				try:
					self.__pin = xinput('enter gpg-passphrase')
				except TclError:
					self.__pin = getpass('enter gpg-passphrase: ')
		except KeyboardInterrupt:
			pass



from os import \
    X_OK as _XOK, \
    access as _access,\
    environ as _environ

def which(prog):
	for path in _environ['PATH'].split(':'):
		if _access('%s/%s'%(path, prog), _XOK):
			return '%s/%s'%(path, prog)



"""pwclip packaging information"""
import sys
from os import getcwd
from os.path import join, dirname

modname = distname = 'pwclip'
numversion = (0, 4, 8)
version = '.'.join([str(num) for num in numversion])
provides = ['pwclip']
install_requires = [
    'pyusb', 'PyYAML', 'argcomplete',
    'python-gnupg', 'python-yubico']
license = 'GPL'
description = "gui to temporarily save passwords to clipboard (paste-buffer)"
web = 'http://janeiskla.de'
mailinglist = ""
author = 'Leon Pelzer'
author_email = 'mail@leonpelzer.de'
download_url = 'https://pypi.python.org/pypi/pwclip/%s#downloads'%version
classifiers = ['Development Status :: 4 - Beta',
               'Environment :: Console',
               'Environment :: MacOS X',
               'Environment :: Win32 (MS Windows)',
               'Environment :: X11 Applications',
               'Intended Audience :: Developers',
               'Intended Audience :: End Users/Desktop',
               'Intended Audience :: System Administrators',
               'Intended Audience :: Information Technology',
               'License :: OSI Approved :: GNU General Public License (GPL)',
               'Operating System :: OS Independent',
               'Programming Language :: Python :: 3',
               'Topic :: Security',
               'Topic :: Utilities',
               'Topic :: Desktop Environment',
               'Topic :: System :: Systems Administration']

try:
	with open(join(getcwd(), 'README'), 'r') as rfh:
		readme = rfh.read()
except FileNotFoundError:
	readme = ''

long_desc = ( readme )

scripts = [join('bin', 'pwclip')]

entry_points = {'console_scripts': ['pwclip = pwclip.__init__:pwclipper']}



#!/usr/bin/env python3
"""ssh connection and remote command """

#global imports"""
import os
import sys
import paramiko

# local relative imports
from socket import getfqdn as fqdn

from colortext import abort

# default vars
__version__ = '0.1'

class SecureSHell(object):
	_dbg = False
	user = 'root'
	host = ''
	def __init__(self, *args, **kwargs):
		for arg in args:
			if hasattr(self, arg):
				setattr(self, arg, True)
			elif hasattr(self, '_%s'%(arg)):
				setattr(self, '_%s'%(arg), True)
		for (key, val) in kwargs.items():
			if hasattr(self, key):
				setattr(self, key, val)
		if self.dbg:
			print('\033[01;30m%s\033[0m'%SecureSHell.__mro__)
			for (key, val) in self.__dict__.items():
				print('\033[01;30m%s = %s\033[0m'%(key, val))
	@property               # dbg <bool>
	def dbg(self):
		return self._dbg
	@dbg.setter
	def dbg(self, val):
		self._dbg = val if val else False

	@staticmethod
	def _ssh_(host, user, port=22):
		ssh = paramiko.SSHClient()
		ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
		try:
			ssh.connect(host, int(port), username=user)
		except paramiko.ssh_exception.SSHException as err:
			print(err)
		return ssh

	def rstdo(self, cmd, host=None, user=None):
		host = host if host else self.host
		user = user if user else self.user
		ssh = self._ssh_(host, user)
		try:
			_, out, _ = ssh.exec_command(cmd)
		except KeyboardInterrupt:
			abort()
		return ''.join(out.readlines())

	def scp(self, src, trg, host=None, user=None):
		user = user if user else self.user
		host = host if host else self.host
		host = fqdn(host)
		ssh = self._ssh_(host, user)
		scp = ssh.open_sftp()
		return scp.put(src, trg)

	def compstats(self, src, trg, host=None, user=None):
		host = host if host else self.host
		user = user if user else self.user
		smt = int(str(int(os.stat(src).st_mtime))[:6])
		rmt = self.rstdo(
            'stat -c %%Y %s'%os.path.basename(src), host=host, user=user)
		if rmt:
			rmt = int(str(rmt)[:6])
		if rmt == smt:
			return
		srctrg = src, '%s@%s:%s'%(user, host, trg)
		if int(rmt) > int(smt):
			srctrg = '%s@%s:%s'%(user, host, trg), src
		return srctrg

	def scpcompstats(self, src, trg, host=None, user=None):
		user = user if user else self.user
		host = host if host else self.host
		smt = int(str(int(os.stat(src).st_mtime))[:6])
		rmt = self.rstdo(
            'stat -c %%Y %s'%os.path.basename(src), host=host, user=user)
		if rmt:
			rmt = int(str(rmt)[:6])
		if rmt == smt:
			return
		elif int(rmt) > int(smt):
			return self.scp(trg, src, host=host, user=user)
		return self.scp(src, trg, host=host, user=user)




if __name__ == '__main__':
	"""module debugging area"""
	#ssh = SecureSHell(**{'host':'bigbox.janeiskla.de'})
	#print(ssh.command('cat /etc/debian_version'))



from sys import \
    stderr as _stderr
__puke = _stderr.write

def userfind(pattern='1000', mode='user'):
	"""
        >>> 0 = user
        >>> 1 = x
        >>> 2 = uid
        >>> 3 = gid
        >>> 4 = comment
        >>> 5 = home
        >>> 6 = shell
	"""
	user = 0
	x = 1
	uid = 2
	gid = 3
	comment = 4
	home = 5
	shell = 6
	mode = int(eval(mode))
	pstr = str(pattern)
	try:
		with open('/etc/passwd', 'r') as pwd:
			try:
				hits = [f.split(':') for f in [l for l in pwd.readlines() if pstr in l] if pstr in f][0]
			except IndexError as err:
				__puke(str(err))
				hits = []
	except PermissionError as err:
		__puke(str(err))
		return err
	if hits:
		return list(hits)[mode]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# This file is free software by d0n <d0n@janeiskla.de>
#
# You can redistribute it and/or modify it under the terms of the GNU -
# Lesser General Public License as published by the Free Software Foundation
#
# This is distributed in the hope that it will be useful somehow.
# !WITHOUT ANY WARRANTY!
#
# Without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
"""pwclip main program"""

# global & stdlib imports
import sys

try:
    from os import fork
except ImportError:
	def fork(): return 0

from os import environ, path

from os.path import isfile, basename, dirname, expanduser

from yaml import load

from argparse import ArgumentParser

from time import sleep

# local relative imports
from colortext import bgre, abort, tabd, fatal

from system import copy, paste, xinput, xnotify

from secrecy import PassCrypt, ykchalres

from pwclip.__pkginfo__ import version

def forkwaitclip(text, poclp, boclp, wait=3):
	if fork() == 0:
		try:
			copy(text, mode='pb')
			sleep(int(wait))
		except KeyboardInterrupt:
			abort()
		finally:
			copy(poclp, mode='p')
			copy(boclp, mode='b')
	exit(0)

def __passreplace(pwlist):
	__pwcom = ['*'*len(pwlist[0])]
	if len(pwlist) > 1:
		__pwcom.append(pwlist[1])
	return __pwcom

def __dictreplace(pwdict):
	__pwdict = {}
	for (usr, ent) in pwdict.items():
		if isinstance(ent, dict):
			__pwdict[usr] = {}
			for (u, e) in ent.items():
				__pwdict[usr][u] = __passreplace(e)
		elif ent:
			__pwdict[usr] = __passreplace(ent)
	return __pwdict


# global default variables
def cli():
	_me = basename(dirname(__file__))
	cfg = expanduser('~/.config/%s.yaml'%_me)
	try:
		with open(cfg, 'r') as cfh:
			cfgs = load(cfh.read())
	except FileNotFoundError:
		cfgs = {}
	pars = ArgumentParser() #add_help=False)
	pars.set_defaults(**cfgs)
	pars.add_argument(
        '--version',
        action='version', version='%(prog)s-v'+version)
	pars.add_argument(
        '-D', '--debug',
        dest='dbg', action='store_true', help='debugging mode')
	pars.add_argument(
        '-A', '--all',
        dest='aal', action='store_true',
        help='switch to all users entrys (instead of current user only)')
	pars.add_argument(
        '-s', '--show-passwords',
        dest='sho', action='store_true',
        help='switch to display passwords (replaced with * by default)')
	pars.add_argument(
        '-a', '--add',
        dest='add', metavar='ENTRY',
        help='add ENTRY (password will be asked interactivly)')
	pars.add_argument(
        '-c', '--change',
        dest='chg', metavar='ENTRY',
        help='change ENTRY (password will be asked interactivly)')
	pars.add_argument(
        '-d', '--delete',
        dest='rms', metavar='ENTRY', nargs='+',
        help='delete ENTRY(s) from the passcrypt list')
	pars.add_argument(
        '-l', '--list',
        nargs='?', default=False,
        dest='lst', metavar='PATTERN',
        help='search entry matching PATTERN if given otherwise list all')
	pars.add_argument(
        '--yaml',
		dest='yml', metavar='YAMLFILE',
		default=path.expanduser('~/.pwd.yaml'),
        help='set location of one-time YAMLFILE to read')
	pars.add_argument(
        '-p', '--passcrypt',
        dest='pcr', metavar='CRYPTFILE',
        default=path.expanduser('~/.passcrypt'),
        help='set location of CRYPTFILE to use for gpg features')
	pars.add_argument(
        '-r', '--recipients',
        dest='rcp', metavar='RECIPIENT',
        help='gpg recipients (identifier) for GPG-Keys to use')
	pars.add_argument(
        '-u', '--user',
        dest='usr', metavar='USER',
        help='query entrys of USER (defaults to current user)')
	pars.add_argument(
        '-y', '--ykserial',
        nargs='?', default=False,
        dest='yks', metavar='SERIAL',
        help='switch to yubikey mode and optionally set SERIAL of yubikey')
	pars.add_argument(
        'time',
        nargs='?', default=3, metavar='seconds', type=int,
        help='time to wait before resetting clip (default is 3 max 3600)')
	args = pars.parse_args()

	pargs = [a for a in [
        'dbg' if args.dbg else None,
        'aal' if args.aal else None,
        'sho' if args.sho else None] if a]
	pkwargs = {}
	if args.pcr:
		pkwargs['crypt'] = args.pcr
	if args.usr:
		pkwargs['user'] = args.usr
	if args.yml:
		pkwargs['plain'] = args.yml
	if args.remote:
		pkwargs['remote'] = args.remote
	if args.reuser:
		pkwargs['reuser'] = args.reuser
	if args.dbg:
		print(bgre(pars))
		print(bgre(tabd(args.__dict__, 2)))
		print(bgre(pkwargs))
	if not isfile(pkwargs['plain']) and \
          not isfile(pkwargs['crypt']) and args.yks is False:
		fatal(
            'if not in yubi mode either yaml input file', pkwargs['plain'],
            'or already written passcrypt', pkwargs['crypt'], 'must exsist')
	poclp, boclp = paste('pb')
	if args.yks is not False:
		args.time = args.yks if args.yks and len(args.yks) < 6 else args.time
		if 'YKSERIAL' in environ.keys():
			ykser = environ['YKSERIAL']
		ykser = args.yks if args.yks and len(args.yks) >= 6 else None
		forkwaitclip(ykchalres(xinput(), ykser=ykser), poclp, boclp, args.time)
		exit(0)
	if args.lst and args.lst.isdigit() and int(args.lst) <= 3600:
		args.time = int(args.lst)
		args.lst = None
	for a in (args.lst, args.add, args.chg):
		if a and len(a) < 2:
			fatal('input', a, 'is too short')

	pcm = PassCrypt(*pargs, **pkwargs)
	if args.lst is not False:
		__ent = pcm.lspw(args.lst)
		if not __ent:
			fatal('could not decrypt')
		elif __ent and args.lst and not __ent[args.lst]:
			fatal('could not find entry for', args.lst, 'in', pkwargs['crypt'])
		elif args.lst and __ent:
			__pc = __ent[args.lst]
			if __pc:
				if len(__pc) == 2:
					xnotify('%s: %s'%(args.lst, __pc[1]), wait=args.time)
				forkwaitclip(__pc[0], poclp, boclp, args.time)
	elif args.add:
		if not pcm.adpw(args.add):
			fatal('could not add entry', args.add)
		__ent = pcm.lspw(args.add)
	elif args.chg:
		if not pcm.chpw(args.chg):
			fatal('could not change entry', args.chg)
		__ent = pcm.lspw(args.chg)
	elif args.rms:
		for r in args.rms:
			if not pcm.rmpw(r):
				fatal('could not delete entry', r)
		__ent = pcm.lspw()
	else:
		__in = xinput()
		if not __in:
			exit(1)
		__ent = pcm.lspw(__in)
		if __ent:
			if not __in in __ent.keys() or not __ent[__in]:
				fatal(
                    'could not find entry for',
                    __in, 'in', pkwargs['crypt'])
			__pc = __ent[__in]
			if __pc:
				if len(__pc) == 2:
					xnotify('%s: %s'%(__in, __pc[1]), args.time)
				forkwaitclip(__pc[0], poclp, boclp, args.time)
	if not args.sho:
		__ent = __dictreplace(__ent)
	print(tabd(__ent))
